#
# Element.tcl
#
# Routines to handle common issues with elements (classes, fields etc)
#

package provide Element 0.1;

class Element {

  extends TclDecorator;

  import Console;

  method hasProperty {property} {
    set p1 [$self get_property "$property"];
    set p2 [$self get_property " $property "];
    return [expr {$p1 == "<unspecified>"} && {$p2 == "<unspecified>"} ? 0: 1]
  }

  method getProperty {property} {
    set p [$self get_property $property];
    if { $p == "<unspecified>" } {
      return "";
    } else {
      return "$p";
    }
  }

  #
  # Sanity checks omitted on purpose to keep the method compact,
  # so it is relied on that e.g. a Class is never tested against "is static".
  #
  method is {aspect} {
    switch "$aspect" {
      abstract     { return [$self get_is_abstract]; }
      static       {
	return [$self get_is_classifier_scope]; # bug in ClassDesigner's docs
      } 
      final        { return [$self hasProperty "final"]; }
      native       { return [$self hasProperty "native"]; }
      synchronized { return [$self hasProperty "synchronized"]; }
      volatile     { return [$self hasProperty "volatile"]; }
      transient    { return [$self hasProperty "transient"]; }
      beanprop     { return [$self hasProperty "beanprop"]; }
      class        { return [$self is_kind_of "Class"]; }
      interface    { return [$self is_kind_of "Interface"]; }
      innerclass   { 
	return [expr {[$self get_implement_as]} == {"innerclass"}];
      }
    }
  }

  method prop {aspect} {
    switch "$aspect" {
      abstract    -
      static      -
      final       -
      native	  -
      syncronized -
      volatile    -
      transient   { 
	if { [$self is $aspect] } { 
 	  return "$aspect "; 
	} else {
	  return "";
	}
      }

      visibility {
	set vis "";
	if { [$self is_kind_of "Feature"] } {
	  if { [$self hasProperty "visibility"] } {
	    set vis [$self getProperty "visibility"];
	  }
	  if { $vis == "" } {
	    set vis [$self get_visibility];
	  }
	} else {
	  set vis [$self get_namespace_visibility];
	}
	if { $vis == "unspecified" } {
	  return "";
	}
	return "$vis ";
      }

      beanprop {
	if { ! [$self hasProperty "beanprop"] } {
	  return "";
	}
	switch [$self get_type] {
	  boolean { set type "boolean" }
	  Boolean { set type "boolean" }
	  default { set type "simple" }
	}
	set mod [$self getProperty "beanprop"];
	if { $mod != "" } {
	  append type "-";
	}
	return "$type$mod";
      }

      exceptions {
        if { [$self hasProperty "exceptions"] } {
          return "[$self getProperty "exceptions"] ";
        } else {
          return "";
        }
      }
    }
  }

  method warning {msg {mode verbose}} {
    if { $mode == "verbose"} {
      Console warning "$msg";
    }
  }

  method error {msg {mode verbose}} {
    if { $mode == "verbose"} {
      Console error "$msg";
    }
  }
}
