#
# Java.tcl
# 
# A TCL Java Code generator for Cayenne ClassDesigner
#

#
# Load library
#

proc sc_platform {} {
  global tcl_platform;

  return $tcl_platform(platform);
}

if { [info proc "Main"] == "Main" } {
  sc_puts "Library already loaded.";
} else {
  sc_puts "Loading library...";

  proc library {} {
    global sc_library;
    return $sc_library;
  }

  set script [info script];
  set dir [file dirname $script];
  
  if { [file exists [file join $dir "TclClass.tcl"]] } {
    set sc_library $dir; 
  } else {
    switch $tcl_platform(platform) {
      windows   { set sc_library "..\codegen"; }
      unix      { set sc_library "../codegen"; }
      macintosh { set sc_library "..:codegen"; }
      default   { return -code error "Unknown platform $tcl_platform(platform)"; }
    }
  }

  if { ! [file exists [file join $sc_library "TclClass.tcl"]] } {
    sc_puts "Can't find library."
    return -code error LibraryNotFound;
  }

  source [file join $sc_library TclClass.tcl];
  source [file join $sc_library TclDecor.tcl];
  source [file join $sc_library String.tcl];
  source [file join $sc_library Indent.tcl];
  source [file join $sc_library Output.tcl];
  source [file join $sc_library Console.tcl];
  source [file join $sc_library Expander.tcl];

  source [file join $sc_library Comment.tcl];
  source [file join $sc_library Section.tcl];
  source [file join $sc_library Element.tcl];
  source [file join $sc_library Attrib.tcl];
  source [file join $sc_library Operat.tcl];
  source [file join $sc_library AssocEnd.tcl];
  source [file join $sc_library Class.tcl]; 
  source [file join $sc_library Interfac.tcl]; 
  source [file join $sc_library Package.tcl];
  source [file join $sc_library Componen.tcl];
  source [file join $sc_library Model.tcl];
  source [file join $sc_library ObjInfc.tcl];

  rename library {};

  sc_puts "Loading library...done.";
  sc_showConsole;
  sc_clearConsole;

}

class Main {

  import Model;
  import Console;

  Model unmark;
  Model mark;
  Model print;
  Console message "Code generation complete.";
}
