#
# Operations
#

package provide Operation 0.1;

class Operation {
 
  extends Element;

  import Output;
  import Comment;
  import Section;
  import Console;
  import library;

  staticarray template;

  field classifier; # either Class or Interface

  source [file join [library] TmplOper.tcl];

  method init {{initializer {}}} {
    set classifier $initializer;
  }

  method is {aspect} {
    if { $classifier != {} && [$classifier is interface] } {
      switch $aspect    {
	abstract    { return 1; } # interface methods are implicitly abstract
      }
    }

    #
    # For convenience reasons, the semantics of the "polymorphic" aspect
    # have been reversed so that "is polymorphic" is treated as "is final".
    # (In Java, methods are "non-final" by default while
    # in UML and e.g. C++ they're "final".)
    #
    if { $aspect == "final" } {
      return [$self get_is_polymorphic];
    }
    return [$self Element.is $aspect];
  }

  method violations {{mode verbose}} {
    set name [$self get_name];
    set num_errors 0;

    if { [$self is abstract] && [$self is final] } {
      $self error "$name: method declared both abstract and final" $mode; 
      incr num_errors;
    }

    return $num_errors;
  }

  method print {class} {
    set name [$self get_name];
    set classname [$class get_name];

    Comment print description "[$self get_description]";

    set expander [new Expander];

    $expander bind visibility   "[$self prop visibility]";
    $expander bind abstract     "[$self prop abstract]";
    $expander bind static       "[$self prop static]";
    $expander bind final        "[$self prop final]";
    $expander bind native       "[$self prop native]";
    $expander bind synchronized "[$self prop synchronized]";

    set result [$self get_result];
    if { [llength $result] > 0 } {
      $expander bind type	"[[lindex $result 0] get_type] ";
    } else {
      $expander bind type	"";	# Constructor
    }

    $expander bind name		"$name";
    $expander bind parameters   "[$self parameters]";
    $expander bind exceptions   "[$self prop exceptions]";

    set hasbody [expr { ! ( [$self is abstract] || [$self is native] ) } ];
    $expander bind hasbody $hasbody;

    Output print [$expander expand $template(begin)];

    if { $hasbody } {
      set newId "${classname}::[$self get_signature]";
      set oldId "[$self get_csid]";
      if { $oldId == ""} {
        set oldId $newId;
      }
      Section print $oldId $newId;
    }

    Output print [$expander expand $template(end)];

    delete $expander;
  }

  method parameters {} {
    set result {};
    set plist [$self get_parameters];
    set param [lindex $plist 0];
    if { $param != {} } {
      set result "[$param get_type] [$param get_name]";
    }
    foreach param [lrange $plist 1 end] {
      append result ", [$param get_type] [$param get_name]";
    }
    return $result;
  }
}
