#
# Section.tcl
#
# Handle code sections
#

package provide Section 0.1;

class Section {

  import Output;
  import Indent;
  import Console;

  staticarray sections;
  staticfield unused;

  staticfield start "// +*************** UNUSED CODE ***************";
  staticfield end   "// -*************** UNUSED CODE ***************";

  staticmethod init {} {
    foreach elm [array names sections] {
      unset sections($elm);
    }
    foreach elm [array names unused] {
      unset unused($elm);
    }
  }

  staticmethod exists {type id} {
    set csid "${type}::$id";
    return [info exists sections($csid)];
  }

  staticmethod print {oldId newId} {
    Output print "//+ $newId";
    Indent more;
    if { [info exists sections($oldId)] } {
      Output verbatim $sections($oldId);
      unset sections($oldId); 
    }
    Indent less;
    Output print "//- $newId";
  }

  staticmethod printUnused {} {
    Output print "$start";
    Output verbatim "$unused";
    foreach id [array names sections] {
      Output print "//+ $id";
      Output verbatim "$sections($id)";
      Output print "//- $id";
    }
    Output print $end;
  }

  staticmethod parseMarker {line} {

    set rexp {^ *//[+-] ([A-Za-z][A-Za-z0-9_]*::[A-Za-z0-9_]*\([][ \.,A-Za-z0-9_]*\)+)$}
    if {[regexp $rexp $line all result]} {
      return "$result";
    } 
    return "";
  }
    
  staticmethod extract {filename} {
    
    # Null any previous unused code
    set unused "";

    # Open the file
    if { [file exists $filename] == 0 } return;
    set input [open $filename r];
    Console message "Reading $filename ...";

    # Extract code sections
    while { [gets $input line] != -1 } {
      set id [parseMarker $line];
      if { "$id" != "" } {
        Console message "Reading code for $id";
        set code "";
 	while { [gets $input line] != -1 } {
 	  set id2 [parseMarker $line];
 	  if { "$id2" == "$id" } break;
 	  if { "$id2" != "" } {
 	    Console warning "Section: Section mismatch: $id != $id2";
 	    break;
          }
 	  append code "$line\n";
	}
	set sections($id) $code;
	continue;
      }
      if { "$line" == "$start" } {
	set code "";
	while { [gets $input line] != -1 } {
	  if { "$line" == "$end" } break;
	  append code "$line\n";
        }
	set unused $code;
	continue;
      }
    }
  
    close $input;
  }
}
