#
# TclDecor.tcl
#
# NOTE: Currently the unknown method of TclDecorator is used as
#       the standard procedure for introducing new instances of classes
#       to the code generator.
#       New instances are typically not explicitly created; instead, they
#       get automatically created when referenced for the first time.
#

class TclDecorator {

  proc unknown {arg1 {arg2 {}} args} {

    #
    # 1. Assume that arg1 is an instance name and arg2 is a command;
    #    derive the corresponding class name (e.g. "Class_123" => "Class")
    #

    set lst [split $arg1 _];
    if { [llength $lst] == 2
	&& [string match {[A-Z][A-Za-z]*} [lindex $lst 0]]} {
      set class [lindex $lst 0];

      #
      # Has the class been declared?
      #   If so, create a new instance or alias an instance from another class,
      #          then re-run the command.
      #   If not, run the command on the global level.
      #
      if { [^ info command $class] == $class } {
	if { [^ $class info proc $arg1]  != $arg1 } {
	  ^ $class new $class $arg1;
	} else {
	  .linksym $class $arg1;
	}
	return [eval $arg1 $arg2 $args];
      } else {
	return [^ $arg1 $arg2 $args]; 
      }
    }

    #
    # 2. Assume that arg2 is a class name and arg1 is a command (static method)
    #

    if { [ ^ info command $arg2] == $arg2 } {
      ^ $arg2 $arg1 $args;
    } else {
      eval .unknown $arg1 $args;
    }
  }
}
