#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

#
# Make template file
#
# This tclsh script collects templates stored on separate files
# and constructs a single file from them.  This file can then,
# in turn, be source'd into a ClassDesigner TCL class, e.g. End.tcl
#

if { $argc != 2 || ! [file isdirectory [lindex $argv 0]] } {
  puts stderr "Usage: $argv0 directory outputfile";
  exit 1;
}

set output [open [lindex $argv 1] w];
puts $output "
# This file is automatically generated with $argv0.
#
# Do not modify. Instead, modify the files in the directory [lindex $argv 0]
# and rerun the generating command:
#   $argv0 $argv
#
# Please remember to quote the brackets [ ] when not used with TCL commands.
#";

foreach file [glob [file join [lindex $argv 0] "*.java"]] {
  set name [file rootname [file tail $file]];
  set input [open $file];

  puts $output "###########################################################";
  puts -nonewline $output "set template($name) ";
  puts -nonewline $output [list [read -nonewline $input]];
  puts $output ";";

  close $input;
}

close $output;
