/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Module:  Adaptor
  Package: conduits
  File:    Adaptor.java
*/
package fi.hut.tcm.jacob.conduits;
/**
  Adaptor
  
  An adaptor is a Conduit that connects the conduit graph to the outside 
  world.  For example, a low level conduit graph could be connected to the 
  Ethernet driver through an Ethernet adaptor.
*/
public abstract class Adaptor
  extends Sedaptor
  implements AdaptorEventListener {

  /**
    Creates a new, empty adaptor. Used by subclasses.
  */
  protected Adaptor() {
  
    //+ Adaptor::Adaptor( )
    super();
    //- Adaptor::Adaptor( )
  }
  
  
  /**
  */
  protected final void accept(Visitor visitor, Side from)
    throws ConductingException, IllegalArgumentException  {
  
    //+ Adaptor::accept(Visitor, Side)
    visitor.at(this, from);
    //- Adaptor::accept(Visitor, Side)
  }
  
  
  /**
    Actually  handles an incoming message.
    Must be overridden by subclasses.
  */
  public abstract void adapt(Message message);
  
  
  
  
  /**
  */
  public int getSideCount() {
  
    //+ Adaptor::getSideCount( )
    return A + 1;
    //- Adaptor::getSideCount( )
  }
  
  
  /**
  */
  public Object clone()
    throws CloneNotSupportedException  {
  
    //+ Adaptor::clone( )
    throw new CloneNotSupportedException("must override to support adaptor cloning");
    //- Adaptor::clone( )
  }
  
  
} // Adaptor
