/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Module:  InstallMessenger
  Package: conduits
  File:    InstallMessenger.java
*/

package fi.hut.tcm.jacob.conduits;

/**
  InstallMessenger
  
  Carries Installation information for ConduitFactories.
  When a Message arrives at a ConduitFactory, the factory
  determines if the Message's Messenger is an InstallMessenger.  If so, the
  ConduitFactory queries the message about the exact operation to perform.
  The InstallMessenger is used to perform the query. If not so, the 
  ConduitFactory uses its internal resources.
*/
public interface InstallMessenger
  extends Messenger {

  /**
    Returns a Side (corresponding to <code>index</index>) where the cloned 
    Conduit is to be attached to, or null if the Side is to be determined 
    by the ConduitFactory itself.
    
    @param message message where to look for the Sides
    @param index typically Side.A or Side.B, i.e., index of the side that 
    is to be attached to.
    @return a Side to attach, or null.
  */
  public abstract Side getEndpoint(Message message, int index);
  
  
  
  
  /**
  */
  public abstract int getEndpointCount(Message message);
  
  
  
  
  /**
    Returns a prototype conduit that the ConduitFactory attempts to clone, 
    or null if to use the prototype stored at the factory.
    @param message the message that carries the conduit
    @return a Conduit to clone, or null
    
  */
  public abstract Conduit getPrototype(Message message);
  
  
  
  
} // InstallMessenger


// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
