/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  Message
  Package: conduits
  File:    Message.java
*/
package fi.hut.tcm.jacob.conduits;
import java.util.EventObject;
public final class Message
  extends EventObject {

  /**
  */
  private Object contents;
  
  /**
    Used to cheat EventObject constructor.
  */
  private static final Object o = new Object();
  
  /**
  */
  
  /* ****************** Transports ************************/
  
  transient private MessageTransporter transporter;
  
  /* ****************** End of Transports *****************/
  
  
  /**
  */
  
  /* ****************** Interprets ************************/
  
  transient private Messenger interpreter;
  
  /* ****************** End of Interprets *****************/
  
  
  /**
    A Message may carry OutOfBand data. This association
    allows one to access the OOB information
  */
  
  /* ****************** Oob ************************/
  
  transient private OutOfBand oob;
  
  /* ****************** End of Oob *****************/
  
  
  /**
  */
  Message() {
  //+ Message::Message( )
    // Don't initialize yet
    
    // Must cheat...
    
    super(o);
    source = null;
  //- Message::Message( )
  }
  
  /**
  */
  public static Message newMessage(Object contents, Messenger messenger, OutOfBand oob) {
  //+ Message::newMessage(Object, Messenger, OutOfBand)
    Message message;
    message = (Message)MessageFactory.instance.newObject();
    message.init(contents, messenger, oob);
    return message;
  //- Message::newMessage(Object, Messenger, OutOfBand)
  }
  
  /**
  */
  protected void init(Object contents, Messenger messenger, OutOfBand oob) {
  //+ Message::init(Object, Messenger, OutOfBand)
    setTransporter(null);
    setSourceSide(null);
    setContents(contents);
    setMessenger(messenger);
    setOob(oob);
  //- Message::init(Object, Messenger, OutOfBand)
  }
  
  /**
  */
  protected void finalize() {
  //+ Message::finalize( )
    setTransporter(null);
    setSourceSide(null);
    setContents(null);
    interpreter = null; // Can't set by method
    setOob(null);
  //- Message::finalize( )
  }
  
  /**
  */
  Side getSourceSide() {
  //+ Message::getSourceSide( )
    return (Side)getSource();
  //- Message::getSourceSide( )
  }
  
  /**
  */
  void setSourceSide(Side source) {
  //+ Message::setSourceSide(Side)
    this.source = source;
  //- Message::setSourceSide(Side)
  }
  
  /**
  */
  protected void setTransporter(MessageTransporter transporter) {
  //+ Message::setTransporter(MessageTransporter)
    this.transporter = transporter;
  //- Message::setTransporter(MessageTransporter)
  }
  
  /**
  */
  protected MessageTransporter getTransporter() {
  //+ Message::getTransporter( )
    return this.transporter;
  //- Message::getTransporter( )
  }
  
  /**
  */
  public void setOob(OutOfBand oob) {
  //+ Message::setOob(OutOfBand)
    this.oob = oob;
  //- Message::setOob(OutOfBand)
  }
  
  /**
  */
  public OutOfBand getOob() {
  //+ Message::getOob( )
    return this.oob;
  //- Message::getOob( )
  }
  
  /**
  */
  public void setMessenger(Messenger messenger) {
  //+ Message::setMessenger(Messenger)
    if (messenger == null)
      throw new IllegalArgumentException("null messenger not allowed");
    this.interpreter = messenger;
  //- Message::setMessenger(Messenger)
  }
  
  /**
  */
  public Messenger getMessenger() {
  //+ Message::getMessenger( )
    return interpreter;
  //- Message::getMessenger( )
  }
  
  /**
  */
  public void setContents(Object contents) {
  //+ Message::setContents(Object)
    this.contents = contents;
  //- Message::setContents(Object)
  }
  
  /**
  */
  public Object getContents() {
  //+ Message::getContents( )
    return this.contents;
  //- Message::getContents( )
  }
  
} // Message

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
