/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  MessageTransporter
  Package: conduits
  File:    MessageTransporter.java
*/
package fi.hut.tcm.jacob.conduits;
import fi.hut.tcm.jacob.utility.Storage;
import fi.hut.tcm.jacob.utility.Pool;


public class MessageTransporter
  extends Transporter {

  /**
  */
  private Message message;
  
  /**
  */
  private static final Message sentinel = makeSentinel();
  
  /**
  */
  MessageTransporter() {
  //+ MessageTransporter::MessageTransporter( )
    super();
    setNext(null);
    message = sentinel;   // Must first initialize, 
    setMessage(sentinel); // only then call setMessage()
  //- MessageTransporter::MessageTransporter( )
  }
  
  /**
  */
  public static MessageTransporter newMessageTransporter(Side at, Message message) {
  //+ MessageTransporter::newMessageTransporter(Side, Message)
    MessageTransporter tr;
    tr = (MessageTransporter)MessageTranspFactory.instance.newObject();
    tr.setNext(at);
    tr.setMessage(message);
    return tr;
  //- MessageTransporter::newMessageTransporter(Side, Message)
  }
  
  /**
  */
  protected void finalize() {
  //+ MessageTransporter::finalize( )
    setNext(null);
    setMessage(sentinel);
    MessageTranspFactory.instance.releaseObject(this);
  //- MessageTransporter::finalize( )
  }
  
  /**
  */
  protected void proceed(Side side) {
  //+ MessageTransporter::proceed(Side)
    Side at = message.getSourceSide();
    Side next;
    
    if (at == null) {
      next = null;
    } else if (at instanceof MuxSide) {
      next = ((MuxSide)at).getNext(message);
    } else {
      next = at.getNext();
    } 
    setNext(next);
  //- MessageTransporter::proceed(Side)
  }
  
  /**
    Calls <code>Mux.mux</code> to multiplex or demultiplex the carried 
    message.
  */
  public void at(Mux mux, Side at) {
  //+ MessageTransporter::at(Mux, Side)
    message.setSourceSide(at);
    mux.mux(message);
    super.at(mux, at);
    message.setSourceSide(null);
  //- MessageTransporter::at(Mux, Side)
  }
  
  /**
    Calls <code>Session.apply</code> to let the state to work on the 
    message.
  */
  public void at(Session session, Side at) {
  //+ MessageTransporter::at(Session, Side)
    message.setSourceSide(at);
    session.apply(message);
    super.at(session, at);
    message.setSourceSide(null);
  //- MessageTransporter::at(Session, Side)
  }
  
  /**
    Calls <code>Adaptor.adapt</code> to pass the carried message to the 
    adaptor.
  */
  public void at(Adaptor adaptor, Side at) {
  //+ MessageTransporter::at(Adaptor, Side)
    message.setSourceSide(at);
    adaptor.adapt(message);
    super.at(adaptor, at);
    message.setSourceSide(null);
  //- MessageTransporter::at(Adaptor, Side)
  }
  
  /**
  */
  public void at(ConduitFactory factory, Side at) {
  //+ MessageTransporter::at(ConduitFactory, Side)
    message.setSourceSide(at);
    factory.facture(message);
    super.at(factory, at);
    message.setSourceSide(null);
  //- MessageTransporter::at(ConduitFactory, Side)
  }
  
  /**
  */
  protected void setMessage(Message message) {
  //+ MessageTransporter::setMessage(Message)
    if (message == null)
      throw new IllegalArgumentException("null message");
    this.message.setTransporter(null);
    this.message = message;
    message.setTransporter(this);
  //- MessageTransporter::setMessage(Message)
  }
  
  /**
  */
  public Message getMessage() {
  //+ MessageTransporter::getMessage( )
    return this.message;
  //- MessageTransporter::getMessage( )
  }
  
  /**
  */
  private static Message makeSentinel() {
  //+ MessageTransporter::makeSentinel( )
    Message sentinel = new Message();
    sentinel.init(null, new Messenger() {
    			  public void apply(Message m, State s, Session se) {
    			    throw new IllegalStateException("transporting sentinel message");
    			  }
    			},
    			null);
    return sentinel;
  //- MessageTransporter::makeSentinel( )
  }
  
} // MessageTransporter

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
