/**
   Copyright (c) 1998 Pekka Nikander & Juha Parssinen
   All rights reserved.
   XXX   %Z%%M% version %I% date:%G%
   Module:  NotifyMessenger
   Package: conduits
   File:    NotifyMessenger.java
*/

package fi.hut.tcm.jacob.conduits;

/**
  NotifyMessenger
  
  Interprets notification information, which is stored somewhere in the
  notification message, probably in the out of band area (NotifyOob).

  The Conduit that gets a notification message can use the NotifyMessenger
  referenced by the message to find out if the requested operation
  was successful by checking a status code contained within the message.

  If the requestor has stored references to all the requests it has made
  for which no feedback has been received yet, it will be a simple
  matter to determine which operation has completed by comparing
  the reference returned by getRequest() to the list. If this has not been
  done, the requestor can call getRequest.getMessenger() to get a reference
  to the original messenger, in order to use it to interpret the request
  message.
*/
public interface NotifyMessenger
  extends Messenger {

  public static final int SUCCESS = 0;
  public static final int FAILURE = 1;
  
  /**
     Returns a reference to the request message.
     (The notify message signals completion/failure of the request
     returned.)

     @param message the notification message
  */
  public abstract Message getRequest(Message message);

  /**
     Returns the status code for the completion of the
     request returned by getRequest().

     The used status code may be one of the values
     NotifyMessenger.SUCCESS or NotifyMessenger.FAILURE, but need not be.
     
     @param message the notification message
  */
  public abstract int getStatusCode(Message message);

} // NotifyMessenger


// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
