/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Module:  Session
  Package: conduits
  File:    Session.java
*/
package fi.hut.tcm.jacob.conduits;
/**
  Session
  
  A session represents an (active) protocol session or connection.
  Connectionless protocols (e.g. IP or UDP) typically contain one session 
  per protocol entity (e.g. IP address or UDP port). Connection-oriented 
  protocols (e.g. TCP), on the other hand, typically contain a session for 
  each active connection.
*/
public abstract class Session
  extends Sedaptor
  implements SessionEventListener {

  /**
  */
  private State state;
  
  /**
  */
  protected Session(State initialState) {
  
    //+ Session::Session(State)
    state = initialState;
    //- Session::Session(State)
  }
  
  
  /**
  */
  protected final void accept(Visitor visitor, Side from)
    throws ConductingException, IllegalArgumentException  {
  
    //+ Session::accept(Visitor, Side)
      visitor.at(this, from);  
    //- Session::accept(Visitor, Side)
  }
  
  
  /**
  */
  public void apply(Message message) {
  
    //+ Session::apply(Message)
    // There must always be a messenger
    message.getMessenger().apply(message, state, this);
  //- Session::apply(Message)
  }
  
  
  /**
  */
  public void setState(State state)
    throws IllegalArgumentException  {
  
    //+ Session::setState(State)
    if (state == null) 
      throw new IllegalArgumentException("null state");
    this.state = state;
    //- Session::setState(State)
  }
  
  
} // Session


