/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Module:  Transporter
  Package: conduits
  File:    Transporter.java
*/
package fi.hut.tcm.jacob.conduits;
/**
  Transporter
  
  Transporter is the abstract base class for all Visitors that carry 
  information within the conduit graph.
*/
public abstract class Transporter
  extends Object
  implements Visitor, Runnable {

  /**
  */
  protected Side next;
  
  /**
    Creates a new Transporter. Provided for subclasses, should not be 
    called directly.
  */
  protected Transporter() {
  
    //+ Transporter::Transporter( )
    this(null);
    //- Transporter::Transporter( )
  }
  
  
  /**
  */
  protected Transporter(Side at) {
  
    //+ Transporter::Transporter(Side)
    setNext(at);
    //- Transporter::Transporter(Side)
  }
  
  
  /**
  */
  protected void setNext(Side side) {
  
    //+ Transporter::setNext(Side)
    next = side;
    //- Transporter::setNext(Side)
  }
  
  
  /**
  */
  protected abstract void proceed(Side side);
  
  
  
  
  /**
  */
  protected void start(Side at) {
  
    //+ Transporter::start(Side)
    setNext(at);
    start();
    //- Transporter::start(Side)
  }
  
  
  /**
  */
  protected void start() {
  
    //+ Transporter::start( )
    SimpleScheduler.instance.makeRunnable(this);
    //- Transporter::start( )
  }
  
  
  /**
  */
  public void at(Mux mux, Side at) {
  
    //+ Transporter::at(Mux, Side)
    proceed(at);
    //- Transporter::at(Mux, Side)
  }
  
  
  /**
  */
  public void at(Session session, Side at) {
  
    //+ Transporter::at(Session, Side)
    proceed(at);
    //- Transporter::at(Session, Side)
  }
  
  
  /**
  */
  public void at(Adaptor adaptor, Side at) {
  
    //+ Transporter::at(Adaptor, Side)
    proceed(at);
    //- Transporter::at(Adaptor, Side)
  }
  
  
  /**
  */
  public void at(ConduitFactory factory, Side at) {
  
    //+ Transporter::at(ConduitFactory, Side)
    proceed(at);
    //- Transporter::at(ConduitFactory, Side)
  }
  
  
  /**
  */
  public void run() {
  
    //+ Transporter::run( )
    while (next != null) try {
      next.accept(this);
    } catch (ConductingException e) {
      System.out.println(e.getMessage()); // XXX
    }
    //- Transporter::run( )
  }
  
  
} // Transporter


// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
