/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  Icmp (implementation)
    Package: icmp
    File:    Icmp.java


*/
// package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.*;

/**
    Icmp
    
    <p>The Internet Control Message Protocol. This protocol is a required
    part of any IP implementation, even though it logically is a higher
    level protocol, using IP as its transport.<br>
    This conduit encapsulates the other ICMP modules and effectively
    acts as an interface for other protocols wanting to use the ICMP
    protocol.
*/
public class Icmp
  extends Protocol {
    /**
        Specifies the byte offset of the <code>ICMP</code> type in the
        <code>ICMP</code> message.
    */
    public static int TYPE_OFFSET = 0;
    /**
        Specifies the byte offset of the <code>ICMP</code> code in the
        <code>ICMP</code> message.
    */
    public static int CODE_OFFSET = 1;
    /**
        Specifies the byte offset of the <code>ICMP</code> checksum in the
        <code>ICMP</code> message.
    */
    public static int CHECKSUM_OFFSET = 2;
    /**
        The debug output level (0, 1, 2 or 3). Level 0 is no debug output,
        whereas level 3 prints the most debug output. Level 1 is intended
        to be informational,
        levels 2 and 3 give detailed debug information.
    */
    public static int DebugLevel = 2;

    /**

    */
    public Icmp() {

      // Instances
      IcmpEchoAnswerAdaptor answAdap =
	new IcmpEchoAnswerAdaptor();
      IcmpApplicationAccessor applAcce =
	new IcmpApplicationAccessor();
      Mux applMux =
	new Mux(applAcce);
      IcmpHeaderSession headerSessProto =
	new IcmpHeaderSession(IcmpHeaderState.instance);
      Side[] factProtoSides = new Side[2];
      factProtoSides[Side.A] = applMux.getSide(Side.B);
      factProtoSides[Side.B] = null;  // The installer will know this side.
      ConduitFactory applFact =
	new ConduitFactory(headerSessProto, factProtoSides );
      IcmpTypeAccessor typeAcce =
	new IcmpTypeAccessor();
      Mux typeMux =
	new Mux(typeAcce);
      Terminator typeMuxTerm =
	new Terminator(); 
      Terminator applMuxTerm =
	new Terminator();
      Terminator factTerm =
	new Terminator();
      IcmpChecksumSession sumSess = 
	new IcmpChecksumSession(IcmpChecksumState.instance);

      // Define the conduit structure of the system

      try {
	sides[INSIDE_B].attach(applFact.getSide(Side.B));
	applFact.getSide(Side.B).attach(sides[INSIDE_B]);

	applFact.getSide(Side.A).attach(factTerm.getSide(Side.A));
	applMux.getSide(Side.B).attach(applMuxTerm.getSide(Side.A));

	applMux.getSide(Side.A).attach(typeMux.createSide(new Integer(0)));
	answAdap.getSide(Side.A).attach(typeMux.createSide(new Integer(8)));
	
	typeMux.getSide(Side.A).attach(sumSess.getSide(Side.B));
	typeMux.getSide(Side.B).attach(typeMuxTerm.getSide(Side.A));
	
	sumSess.getSide(Side.A).attach(sides[INSIDE_A]);
	sumSess.getSide(Side.B).attach(typeMux.getSide(Side.A));
	
	sides[INSIDE_A].attach(sumSess.getSide(Side.A));
	
      } catch (SideException e) {
	System.out.println("Icmp failed to initialize: " + e);
	System.exit(2);
      }

      // Set a sensible default messenger, as it doesn't make sense
      // that objects external to the protocol would have to know
      // about the internals of the protocol, i.e. which
      // conduit will be encountered first and what kind of messenger
      // it needs.
      
      (sides[Side.A]).setMessenger(IcmpChecksumFromNetMessenger.instance);

    }


    /**
        Creates a deep copy of this protocol by replacing all instance
        variables with references to <bold>copies</bold> of their contents.
    */
    public void deepcopy() {
    //+ Icmp::deepcopy( )
    //- Icmp::deepcopy( )
    }

}; //Icmp



