/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IcmpApplicationAccessor (implementation)
    Package: icmp
    File:    IcmpApplicationAccessor.java


*/
// package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IcmpApplicationAccessor
    
    <p>This accessor examines the ICMP type of the message. The key is the
    same as the ICMP type for all ICMP types except ICMP Echo and ICMP Echo
    Reply. The reason for this is that incoming and outgoing Echo Requests
    (and replies) have different routes.
*/
public class IcmpApplicationAccessor
  implements Accessor {

    private int next_ident = 1;

    /**
        Gets the key of a <code>Message</code>, in order to be able to
        route it correctly. In this version though, it does not yet do it,
        only outputs debug information.
        
        @param    message  which <code>Message</code>
        @return   returns the corresponding key
    */
    public Object getKey(Message message) {
    //+ IcmpApplicationAccessor::getKey(Message)
      int ident = 567;
      Messenger messenger = message.getMessenger();
      if (messenger instanceof InstallMessenger) {
	// XXX DEBUG
	System.out.println("IcmpApplicationAccessor.getKey: InstallMessenger detected!");
	InstallMessenger im = (InstallMessenger)messenger;
	ident = (next_ident = (next_ident + 1) % 0x10000);
	if (Icmp.DebugLevel > 2) {  // #Debug#
	  System.out.println("In IcmpApplicationAccessor.getKey, assigned key is " + ident);
	}
      } else {
	Object buf = message.getContents();
	int len = IcmpUtilityMessenger.instance.getLength(buf); 
	ident = IcmpUtilityMessenger.instance.getShort(buf, 2);
	// DEBUG
	//ident = 789;
	if (Icmp.DebugLevel > 2) {  // #Debug#
	  System.out.println("In IcmpApplicationAccessor.getKey, key is " + ident);
	}
	message.setMessenger(IcmpHeaderFromNetMessenger.instance); //!!
      }
      return new Integer(ident);
    //- IcmpApplicationAccessor::getKey(Message)
    }

    /**
        Sets the key for a message, but right now, does nothing but print
        debug output.
        
        @param    message  which <code>Message</code>
        @param    key  the key for the <code>Message</code>
    */
    public void setKey(Message message, Object key) {
    //+ IcmpApplicationAccessor::setKey(Message, Object)
      Object buf = message.getContents();

      int ident = ((Integer) key).intValue();
      IcmpUtilityMessenger.instance.setShort(buf, 2, ident);

    //- IcmpApplicationAccessor::setKey(Message, Object)
    }


    /**

    */
    public void unsetKey(Message message) {
    //+ IcmpApplicationAccessor::unsetKey(Message)
      // There is nothing to do here.
    //- IcmpApplicationAccessor::unsetKey(Message)
    }

}; //IcmpApplicationAccessor

