/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IcmpApplicationAdaptor (implementation)
    Package: icmp
    File:    IcmpApplicationAdaptor.java


*/
// package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.SimpleScheduler;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IcmpApplicationAdaptor
    
    <p>Creates ICMP ECHO REQUEST messages and receives echo replies.
*/
public class IcmpApplicationAdaptor
  extends Adaptor {

    public IcmpApplicationAdaptor() {
      // Note that we cannot do the connect() stuff here:
      // This adaptor is not yet attached to the Icmp protocol!
    }
    
    public IcmpApplicationAdaptor connect() {
      // We'll have to send an Installer to the ICMP protocol,
      // requesting service.

      IcmpApplicationAdaptor workerAdaptor = new IcmpApplicationAdaptor();

      Side[] oobsides = new Side[2];
      oobsides[Side.A] = null;   // Factory/mux fills in this side
      oobsides[Side.B] = workerAdaptor.getSide(Side.A);
      InstallOob ioob = new InstallOob( oobsides, null);  // no prototype
      Message msg =
      	Message.newMessage(null, Installer.instance, ioob);
      if (sides[Side.A] != null) {
	MessageTransporter mt =
	  MessageTransporter.newMessageTransporter(this.sides[Side.A].getAttached(), msg);
	SimpleScheduler.instance.makeRunnable(mt);
      }
      // In the future, we will wait for a reply before
      // returning "OK" to the caller. Now we assume everything
      // went fine:
      Thread.currentThread().yield();
      return workerAdaptor;
    }

    /**
        Receives replies to echo requests.
        
        @param message The <code>Message</code> carrying the echo reply
    */
    public void adapt(Message message) {
    //+ IcmpApplicationAdaptor::adapt(Message)

      System.out.println("Now in IcmpApplicationAdaptor.adapt()");

      // Incoming replies to packets we have sent should arrive here.

      Messenger messenger = message.getMessenger();
      // We should probably change Adaptor.adapt() so that it would
      // by default just ignore notification messages, just forwarding
      // them as is. Maybe the code that handles that should be
      // in a protected method, which could be used by subclassers
      // if they are not interested in these messages.
      if(messenger instanceof NotifyMessenger) {
	System.out.println("Got notification, status " +
			   ((NotifyOob)message.getOob()).getStatusCode());
	return;
      }

      Object buf = message.getContents();
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      int len = IcmpUtilityMessenger.instance.getLength(buf);
      
      int src = oob.getSrcIpAddr();
      int dst = oob.getDestIpAddr();
      int ident = IcmpUtilityMessenger.instance.getShort(buf, 2);  // cms: updated
      int seq = IcmpUtilityMessenger.instance.getShort(buf, 3);    // cms: updated
      
      System.out.println("<-- Echo Reply: src = " + IpAddress.toString(src) +
			 ", dst = " + IpAddress.toString(dst));
      System.out.println("    ident = " + ident + ", seq = " + seq);
      if (Icmp.DebugLevel > 1) {  // #Debug#
	System.out.println("    Packet dump:");
	// hexDump(buf);
	System.out.println("hexDump not supported yet");
      }
      
    //- IcmpApplicationAdaptor::adapt(Message)
    }

    /**
        Send an ICMP ECHO REQUEST message.
        
        @param    dstIP  destination IP
        @param    ident  packet identfier
        @param    seq  packet sequence number
    */
    public void send(int dstIp, int seq) {
    //+ IcmpApplicationAdaptor::send(int, int)

    // Maximum size for IP header is 60 octets. (RFC 791).
    // Allow for that, plus some extra header room for ethernet
    // and unforseen requirements. = total of 96 octets.
    // ICMP header for Echo messages is 8 octets + optional data (we use 24)
      // Variables: newBinaryMessage(MAXSIZE, INITIALOFFSET, INITIALLENGTH);
      Object buf =
	BufferMessenger.newBuffer(256, 96, 32);

      IcmpUtilityMessenger.instance.setByte(buf, Icmp.TYPE_OFFSET, 8);
      IcmpUtilityMessenger.instance.setByte(buf, Icmp.CODE_OFFSET, 0);

      // data[2] and data[3] are the checksum fields

      //     IcmpUtilityMessenger.instance.setShort(buf, 2, ident);  // cms: updated
      IcmpUtilityMessenger.instance.setShort(buf, 3, seq);  // cms: updated
      IcmpUtilityMessenger.instance.setInt(buf, 2, 0x01020304); // cms: updated these also
      IcmpUtilityMessenger.instance.setInt(buf, 3, 0x05060708);
      IcmpUtilityMessenger.instance.setInt(buf, 4, 0x090a0b0c);
      IcmpUtilityMessenger.instance.setInt(buf, 5, 0x0d0e0f10);
      IcmpUtilityMessenger.instance.setInt(buf, 6, 0x11121314);
      IcmpUtilityMessenger.instance.setInt(buf, 7, 0x15161718);
      // buf.setByte(32, 0x19);
      
      TcpipOutOfBand oob = new TcpipOutOfBand();
      oob.setDestIpAddr(dstIp);

      oob.setSrcIpAddr(0);  // Let the IP stack figure out our address.

      Message msg =
	Message.newMessage(buf, IcmpHeaderToNetMessenger.instance, oob);

      if (sides[Side.A] != null) {
	MessageTransporter mt =
	  MessageTransporter.newMessageTransporter(sides[Side.A].getAttached(),
						   msg);
	SimpleScheduler.instance.makeRunnable(mt);
      }
    //- IcmpApplicationAdaptor::send(int, int)
    }

}; //IcmpApplicationAdaptor


